/**********************************************************************
 * Copyright (c) 2008 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: tptpNativeCall.h,v 1.3 2008/01/30 22:43:37 jcayne Exp $
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/

#include "tptpJSSE.h"

#include "tptp/SocketTL.h"
#include "tptp/TransportSupport.h"

#include "tptp/tptpSSLProvider.h"


int nativeAccept();
void nativeClose(int connectionID);
int nativeHandshake(int connectionID);
int nativeInit(log_service_t *_logService);
int nativeRead(int connectionID, char* buffer, int length);
int nativeShutdown();
void nativeWrite(int connectionID, char* buffer, int length);
